@@ -23,10 +23,8 @@
 
 #define PY_SSIZE_T_CLEAN
 #include <Python.h>
-#if defined(__APPLE__)
+#if defined(__APPLE__) || defined(__linux__)
 #include <sys/xattr.h>
-#elif defined(__linux__)
-#include <attr/xattr.h>
 #endif
 #include <stdio.h>
 
@@ -235,7 +233,7 @@ static inline int _setxattr(const char *path, const char *name, const void *valu
     return setxattr(path, name, value, size, 0, flags);
 }
 static inline int _lsetxattr(const char *path, const char *name, const void *value, size_t size, int flags) {
-    return setxattr(path, name, value, size, 0, flags & XATTR_NOFOLLOW);
+    return setxattr(path, name, value, size, 0, flags | XATTR_NOFOLLOW);
 }
 static inline int _fsetxattr(int filedes, const char *name, const void *value, size_t size, int flags) {
     return fsetxattr(filedes, name, value, size, 0, flags);
@@ -275,41 +273,58 @@ typedef ssize_t (*buf_getter)(target_t *tgt, const char *name,
 
 static ssize_t _list_obj(target_t *tgt, const char *unused, void *list,
                          size_t size) {
+    ssize_t ret;
+
+    Py_BEGIN_ALLOW_THREADS;
     if(tgt->type == T_FD)
-        return _flistxattr(tgt->fd, list, size);
+        ret = _flistxattr(tgt->fd, list, size);
     else if (tgt->type == T_LINK)
-        return _llistxattr(tgt->name, list, size);
+        ret = _llistxattr(tgt->name, list, size);
     else
-        return _listxattr(tgt->name, list, size);
+        ret = _listxattr(tgt->name, list, size);
+    Py_END_ALLOW_THREADS;
+    return ret;
 }
 
 static ssize_t _get_obj(target_t *tgt, const char *name, void *value,
                         size_t size) {
+    ssize_t ret;
+    Py_BEGIN_ALLOW_THREADS;
     if(tgt->type == T_FD)
-        return _fgetxattr(tgt->fd, name, value, size);
+        ret = _fgetxattr(tgt->fd, name, value, size);
     else if (tgt->type == T_LINK)
-        return _lgetxattr(tgt->name, name, value, size);
+        ret = _lgetxattr(tgt->name, name, value, size);
     else
-        return _getxattr(tgt->name, name, value, size);
+        ret = _getxattr(tgt->name, name, value, size);
+    Py_END_ALLOW_THREADS;
+    return ret;
 }
 
 static int _set_obj(target_t *tgt, const char *name,
                     const void *value, size_t size, int flags) {
+    int ret;
+    Py_BEGIN_ALLOW_THREADS;
     if(tgt->type == T_FD)
-        return _fsetxattr(tgt->fd, name, value, size, flags);
+        ret = _fsetxattr(tgt->fd, name, value, size, flags);
     else if (tgt->type == T_LINK)
-        return _lsetxattr(tgt->name, name, value, size, flags);
+        ret = _lsetxattr(tgt->name, name, value, size, flags);
     else
-        return _setxattr(tgt->name, name, value, size, flags);
+        ret = _setxattr(tgt->name, name, value, size, flags);
+    Py_END_ALLOW_THREADS;
+    return ret;
 }
 
 static int _remove_obj(target_t *tgt, const char *name) {
+    int ret;
+    Py_BEGIN_ALLOW_THREADS;
     if(tgt->type == T_FD)
-        return _fremovexattr(tgt->fd, name);
+        ret = _fremovexattr(tgt->fd, name);
     else if (tgt->type == T_LINK)
-        return _lremovexattr(tgt->name, name);
+        ret = _lremovexattr(tgt->name, name);
     else
-        return _removexattr(tgt->name, name);
+        ret = _removexattr(tgt->name, name);
+    Py_END_ALLOW_THREADS;
+    return ret;
 }
 
 /* Perform a get/list operation with appropriate buffer size,
@@ -642,11 +657,7 @@ get_all(PyObject *self, PyObject *args, PyObject *keywds)
         /* Now retrieve the attribute value */
         nval = _generic_get(_get_obj, &tgt, s, &buf_val, &nalloc, &io_errno);
         if (nval == -1) {
-          if (
-#ifdef ENODATA
-              io_errno == ENODATA ||
-#endif
-              io_errno == ENOATTR) {
+          if (io_errno == ENODATA) {
             PyErr_Clear();
             continue;
           } else {
@@ -1173,8 +1184,7 @@ static char __xattr_doc__[] = \
     "   a :exc:`EnvironmentError`; under\n"
     "   Linux, the following ``errno`` values are used:\n"
     "\n"
-    "   - ``ENOATTR`` and ``ENODATA`` mean that the attribute name is\n"
-    "     invalid\n"
+    "   - ``ENODATA`` means that the attribute name is\n invalid\n"
     "   - ``ENOTSUP`` and ``EOPNOTSUPP`` mean that the filesystem does not\n"
     "     support extended attributes, or that the namespace is invalid\n"
     "   - ``E2BIG`` mean that the attribute value is too big\n"
